@@ -157,12 +157,13 @@ try
     uint16_t len;
     if(!getMsgLen(fd, &len))
       break;
-    char msg[len];
-    readn2(fd, msg, len);
+    string msg;
+    msg.resize(len);
+    readn2(fd, msg.data(), len);
     
-    string line(msg, len);
+    string line;
     try {
-      line = sodDecryptSym(line, g_key, readingNonce);
+      line = sodDecryptSym(msg, g_key, readingNonce);
     }
     catch (std::runtime_error& e) {
       errlog("Could not decrypt client command: %s", e.what());
@@ -277,9 +278,9 @@ void doClient(ComboAddress server, const std::string& command)
     writen2(fd, msg);
     uint16_t len;
     getMsgLen(fd, &len);
-    char resp[len];
-    readn2(fd, resp, len);
-    msg.assign(resp, len);
+    msg.clear();
+    msg.resize(len);
+    readn2(fd, msg.data(), len);
     msg=sodDecryptSym(msg, g_key, readingNonce);
     cout<<msg<<endl;
     close(fd);
@@ -317,12 +318,12 @@ void doClient(ComboAddress server, const std::string& command)
     string msg=sodEncryptSym(line, g_key, writingNonce);
     putMsgLen(fd, msg.length());
     writen2(fd, msg);
-    uint16_t len;
+    uint16_t len{0};
     getMsgLen(fd, &len);
-    char resp[len];
-    readn2(fd, resp, len);
-    msg.assign(resp, len);
-    msg=sodDecryptSym(msg, g_key, readingNonce);
+    string resp;
+    resp.resize(len);
+    readn2(fd, resp.data(), len);
+    msg=sodDecryptSym(resp, g_key, readingNonce);
     cout<<msg<<endl;
   }
 }
