@@ -86,6 +86,12 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
   setupCommonLua(client, multi_lua, c_lua, config);
   setupWforceCommonLua(client, multi_lua, c_lua, launchWork);
 
+  if (!multi_lua) {
+    c_lua.writeFunction("setJA3AttrName", [](const std::string& name) {
+      g_ja3_attrname = name;
+    });
+  }
+
   if (!multi_lua && !client) {
     c_lua.writeFunction("addReportSink", [](const std::string& address) {
       ComboAddress ca;
@@ -589,6 +595,14 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       g_bl_db.addEntry(ca, login, seconds, reason);
     });
 
+    c_lua.writeFunction("blacklistJA3", [](const std::string& ja3, unsigned int seconds, const std::string& reason) {
+      g_bl_db.addJA3Entry(ja3, seconds, reason);
+    });
+
+    c_lua.writeFunction("blacklistIPJA3", [](const ComboAddress& ca, const std::string& ja3, unsigned int seconds, const std::string& reason) {
+      g_bl_db.addIPJA3Entry(ca, ja3, seconds, reason);
+    });
+
     c_lua.writeFunction("unblacklistNetmask", [](const Netmask& nm) {
       g_bl_db.deleteEntry(nm);
     });
@@ -605,6 +619,14 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       g_bl_db.deleteEntry(ca, login);
     });
 
+    c_lua.writeFunction("unblacklistJA3", [](const std::string& ja3) {
+      g_bl_db.deleteJA3Entry(ja3);
+    });
+
+    c_lua.writeFunction("unblacklistIPJA3", [](const ComboAddress& ca, const std::string& ja3) {
+      g_bl_db.deleteIPJA3Entry(ca, ja3);
+    });
+
     c_lua.writeFunction("checkBlacklistIP", [](const ComboAddress& ca) {
       return g_bl_db.checkEntry(ca);
     });
@@ -617,6 +639,14 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       return g_bl_db.checkEntry(ca, login);
     });
 
+    c_lua.writeFunction("checkBlacklistJA3", [](const std::string& ja3) {
+      return g_bl_db.checkJA3Entry(ja3);
+    });
+
+    c_lua.writeFunction("checkBlacklistIPJA3", [](const ComboAddress& ca, const std::string& ja3) {
+      return g_bl_db.checkIPJA3Entry(ca, ja3);
+    });
+
     c_lua.writeFunction("getIPBlacklist", []() {
       auto bl = g_bl_db.getIPEntries();
       return getWLBLKeys(bl, "ip");
@@ -629,6 +659,15 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       auto bl = g_bl_db.getIPLoginEntries();
       return getWLBLKeys(bl, "iplogin");
     });
+    c_lua.writeFunction("getJA3Blacklist", []() {
+      auto bl = g_bl_db.getJA3Entries();
+      return getWLBLKeys(bl, "ja3");
+    });
+    c_lua.writeFunction("getIPLoginBlacklist", []() {
+      auto bl = g_bl_db.getIPJA3Entries();
+      return getWLBLKeys(bl, "ipja3");
+    });
+
     c_lua.writeFunction("getBlacklistIPRetMsg", []() {
       return g_bl_db.getIPRetMsg();
     });
@@ -638,6 +677,12 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
     c_lua.writeFunction("getBlacklistIPLoginRetMsg", []() {
       return g_bl_db.getIPLoginRetMsg();
     });
+    c_lua.writeFunction("getBlacklistJA3RetMsg", []() {
+      return g_bl_db.getJA3RetMsg();
+    });
+    c_lua.writeFunction("getBlacklistIPJA3RetMsg", []() {
+      return g_bl_db.getIPJA3RetMsg();
+    });
   }
   else {
     c_lua.writeFunction("blacklistNetmask", [](const Netmask& nm, unsigned int seconds, const std::string& reason) {
@@ -652,6 +697,12 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
     c_lua.writeFunction("blacklistIPLogin", [](const ComboAddress& ca, const std::string& login, unsigned int seconds, const std::string& reason) {
     });
 
+    c_lua.writeFunction("blacklistJA3", [](const std::string& login, unsigned int seconds, const std::string& reason) {
+    });
+
+    c_lua.writeFunction("blacklistIPJA3", [](const ComboAddress& ca, const std::string& login, unsigned int seconds, const std::string& reason) {
+    });
+
     c_lua.writeFunction("unblacklistNetmask", [](const Netmask& nm) {
     });
 
@@ -664,18 +715,32 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
     c_lua.writeFunction("unblacklistIPLogin", [](const ComboAddress& ca) {
     });
 
+    c_lua.writeFunction("unblacklistJA3", [](const std::string& login) {
+    });
+
+    c_lua.writeFunction("unblacklistIPJA3", [](const ComboAddress& ca) {
+    });
+
     c_lua.writeFunction("checkBlacklistIP", [](const ComboAddress& ca) {});
 
     c_lua.writeFunction("checkBlacklistLogin", [](const std::string& login) {});
 
     c_lua.writeFunction("checkBlacklistIPLogin", [](const ComboAddress& ca, const std::string& login) {});
 
+    c_lua.writeFunction("checkBlacklistJA3", [](const std::string& login) {});
+
+    c_lua.writeFunction("checkBlacklistIPJA3", [](const ComboAddress& ca, const std::string& login) {});
+
     c_lua.writeFunction("getIPBlacklist", []() {});
     c_lua.writeFunction("getLoginBlacklist", []() {});
     c_lua.writeFunction("getIPLoginBlacklist", []() {});
+    c_lua.writeFunction("getJA3Blacklist", []() {});
+    c_lua.writeFunction("getIPJA3Blacklist", []() {});
     c_lua.writeFunction("getBlacklistIPRetMsg", []() {});
     c_lua.writeFunction("getBlacklistLoginRetMsg", []() {});
     c_lua.writeFunction("getBlacklistIPLoginRetMsg", []() {});
+    c_lua.writeFunction("getBlacklistJA3RetMsg", []() {});
+    c_lua.writeFunction("getBlacklistIPJA3RetMsg", []() {});
   }
 
   if (!multi_lua) {
@@ -700,6 +765,12 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
     c_lua.writeFunction("setBlacklistIPLoginRetMsg", [](const std::string& msg) {
       g_bl_db.setIPLoginRetMsg(msg);
     });
+    c_lua.writeFunction("setBlacklistJA3RetMsg", [](const std::string& msg) {
+      g_bl_db.setJA3RetMsg(msg);
+    });
+    c_lua.writeFunction("setBlacklistIPJA3RetMsg", [](const std::string& msg) {
+      g_bl_db.setIPJA3RetMsg(msg);
+    });
   }
   else {
     c_lua.writeFunction("blacklistPersistDB", [](const std::string& ip, unsigned int port) {});
@@ -711,6 +782,8 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
     c_lua.writeFunction("setBlacklistIPRetMsg", [](const std::string& msg) {});
     c_lua.writeFunction("setBlacklistLoginRetMsg", [](const std::string& msg) {});
     c_lua.writeFunction("setBlacklistIPLoginRetMsg", [](const std::string& msg) {});
+    c_lua.writeFunction("setBlacklistJA3RetMsg", [](const std::string& msg) {});
+    c_lua.writeFunction("setBlacklistIPJA3RetMsg", [](const std::string& msg) {});
   }
   // End blacklists
 
@@ -741,6 +814,14 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       g_wl_db.addEntry(ca, login, seconds, reason);
     });
 
+    c_lua.writeFunction("whitelistJA3", [](const std::string& ja3, unsigned int seconds, const std::string& reason) {
+      g_wl_db.addJA3Entry(ja3, seconds, reason);
+    });
+
+    c_lua.writeFunction("whitelistIPJA3", [](const ComboAddress& ca, const std::string& ja3, unsigned int seconds, const std::string& reason) {
+      g_wl_db.addIPJA3Entry(ca, ja3, seconds, reason);
+    });
+
     c_lua.writeFunction("unwhitelistNetmask", [](const Netmask& nm) {
       g_wl_db.deleteEntry(nm);
     });
@@ -757,6 +838,14 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       g_wl_db.deleteEntry(ca, login);
     });
 
+    c_lua.writeFunction("unwhitelistJA3", [](const std::string& ja3) {
+      g_wl_db.deleteJA3Entry(ja3);
+    });
+
+    c_lua.writeFunction("unwhitelistIPJA3", [](const ComboAddress& ca, const std::string& ja3) {
+      g_wl_db.deleteIPJA3Entry(ca, ja3);
+    });
+
     c_lua.writeFunction("checkWhitelistIP", [](const ComboAddress& ca) {
       return g_wl_db.checkEntry(ca);
     });
@@ -769,6 +858,14 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       return g_wl_db.checkEntry(ca, login);
     });
 
+    c_lua.writeFunction("checkWhitelistJA3", [](const std::string& ja3) {
+      return g_wl_db.checkJA3Entry(ja3);
+    });
+
+    c_lua.writeFunction("checkWhitelistIPJA3", [](const ComboAddress& ca, const std::string& ja3) {
+      return g_wl_db.checkIPJA3Entry(ca, ja3);
+    });
+
     c_lua.writeFunction("getIPWhitelist", []() {
       auto wl = g_wl_db.getIPEntries();
       return getWLBLKeys(wl, "ip");
@@ -781,6 +878,14 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
       auto wl = g_wl_db.getIPLoginEntries();
       return getWLBLKeys(wl, "iplogin");
     });
+    c_lua.writeFunction("getJA3Whitelist", []() {
+      auto wl = g_wl_db.getJA3Entries();
+      return getWLBLKeys(wl, "ja3");
+    });
+    c_lua.writeFunction("getIPJA3Whitelist", []() {
+      auto wl = g_wl_db.getIPJA3Entries();
+      return getWLBLKeys(wl, "ipja3");
+    });
 
   }
   else {
@@ -796,6 +901,12 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
     c_lua.writeFunction("whitelistIPLogin", [](const ComboAddress& ca, const std::string& login, unsigned int seconds, const std::string& reason) {
     });
 
+    c_lua.writeFunction("whitelistJA3", [](const std::string& login, unsigned int seconds, const std::string& reason) {
+    });
+
+    c_lua.writeFunction("whitelistIPJA3", [](const ComboAddress& ca, const std::string& login, unsigned int seconds, const std::string& reason) {
+    });
+
     c_lua.writeFunction("unwhitelistNetmask", [](const Netmask& nm) {
     });
 
@@ -808,15 +919,27 @@ vector<std::function<void(void)>> setupLua(bool client, bool multi_lua, LuaConte
     c_lua.writeFunction("unwhitelistIPLogin", [](const ComboAddress& ca) {
     });
 
+    c_lua.writeFunction("unwhitelistJA3", [](const std::string& login) {
+    });
+
+    c_lua.writeFunction("unwhitelistIPJA3", [](const ComboAddress& ca) {
+    });
+
     c_lua.writeFunction("checkWhitelistIP", [](const ComboAddress& ca) {});
 
     c_lua.writeFunction("checkWhitelistLogin", [](const std::string& login) {});
 
     c_lua.writeFunction("checkWhitelistIPLogin", [](const ComboAddress& ca, const std::string& login) {});
 
+    c_lua.writeFunction("checkWhitelistJA3", [](const std::string& login) {});
+
+    c_lua.writeFunction("checkWhitelistIPJA3", [](const ComboAddress& ca, const std::string& login) {});
+
     c_lua.writeFunction("getIPWhitelist", []() {});
     c_lua.writeFunction("getLoginWhitelist", []() {});
     c_lua.writeFunction("getIPLoginWhitelist", []() {});
+    c_lua.writeFunction("getJA3Whitelist", []() {});
+    c_lua.writeFunction("getIPJA3Whitelist", []() {});
   }
 
   if (!multi_lua) {
