@@ -79,6 +79,8 @@ syncData g_sync_data;
 WforceReplication g_replication;
 curlTLSOptions g_curl_tls_options;
 
+string g_ja3_attrname = "ja3";
+
 struct SiblingQueueItem {
   std::string msg;
   ComboAddress remote;
@@ -177,12 +179,15 @@ try
     uint16_t len{0};
     if(!getMsgLen(fd, &len))
       break;
-    char msg[len];
-    readn2(fd, msg, len);
-    
-    string line(msg, len);
+    std::vector<char> msgv;
+
+    string msg;
+    msg.resize(len);
+    readn2(fd, msg.data(), len);
+
+    string line;
     try {
-      line = sodDecryptSym(line, key, readingNonce);
+      line = sodDecryptSym(msg, key, readingNonce);
     }
     catch (std::runtime_error& e) {
       errlog("Could not decrypt client command: %s", e.what());
@@ -299,9 +304,9 @@ void doClient(ComboAddress server, const std::string& command)
     writen2(fd, msg);
     uint16_t len{0};
     getMsgLen(fd, &len);
-    char resp[len];
-    readn2(fd, resp, len);
-    msg.assign(resp, len);
+    msg.clear();
+    msg.resize(len);
+    readn2(fd, msg.data(), len);
     msg=sodDecryptSym(msg, key, readingNonce);
     cout<<msg<<endl;
     close(fd);
@@ -341,9 +346,9 @@ void doClient(ComboAddress server, const std::string& command)
     writen2(fd, msg);
     uint16_t len{0};
     getMsgLen(fd, &len);
-    char resp[len];
-    readn2(fd, resp, len);
-    msg.assign(resp, len);
+    msg.clear();
+    msg.resize(len);
+    readn2(fd, msg.data(), len);
     msg=sodDecryptSym(msg, key, readingNonce);
     cout<<msg<<endl;
   }
@@ -786,11 +791,11 @@ char* my_generator(const char* text, int state)
       "controlSocket",
       "stats()",
       "siblings()",
-      "newCA",
-      "newNetmaskGroup",
+      "newCA(",
+      "newNetmaskGroup(",
       "makeKey",
-      "setKey",
-      "testCrypto",
+      "setKey(",
+      "testCrypto(",
       "showWebHooks()",
       "showCustomWebHooks()",
       "showCustomEndpoints()",
@@ -807,41 +812,53 @@ char* my_generator(const char* text, int state)
       "setNumWebHookThreads(",
       "blacklistPersistDB(",
       "blacklistPersistReplicated()",
-      "blacklistNetmask",
-      "blacklistIP",
-      "blacklistLogin",
-      "blacklistIPLogin",
-      "unblacklistNetmask",
-      "unblacklistIP",
-      "unblacklistLogin",
-      "unblacklistIPLogin",
-      "checkBlacklistIP",
-      "checkBlacklistLogin",
-      "checkBlacklistIPLogin",
+      "blacklistNetmask(",
+      "blacklistIP(",
+      "blacklistLogin(",
+      "blacklistIPLogin(",
+      "blacklistJA3(",
+      "blacklistIPJA3(",
+      "unblacklistNetmask(",
+      "unblacklistIP(",
+      "unblacklistLogin(",
+      "unblacklistIPLogin(",
+      "unblacklistJA3(",
+      "unblacklistIPJA3(",
+      "checkBlacklistIP(",
+      "checkBlacklistLogin(",
+      "checkBlacklistIPLogin(",
+      "checkBlacklistJA3(",
+      "checkBlacklistIPJA3(",
       "whitelistlistPersistDB(",
       "whitelistPersistReplicated()",
-      "whitelistNetmask",
-      "whitelistIP",
-      "whitelistLogin",
-      "whitelistIPLogin",
-      "unwhitelistNetmask",
-      "unwhitelistIP",
-      "unwhitelistLogin",
-      "unwhitelistIPLogin",
-      "checkWhitelistIP",
-      "checkWhitelistLogin",
-      "checkWhitelistIPLogin",
+      "whitelistNetmask(",
+      "whitelistIP(",
+      "whitelistLogin(",
+      "whitelistIPLogin(",
+      "whitelistJA3(",
+      "whitelistIPJA3(",
+      "unwhitelistNetmask(",
+      "unwhitelistIP(",
+      "unwhitelistLogin(",
+      "unwhitelistIPLogin(",
+      "unwhitelistJA3(",
+      "unwhitelistIPJA3(",
+      "checkWhitelistIP(",
+      "checkWhitelistLogin(",
+      "checkWhitelistIPLogin(",
+      "checkWhitelistJA3(",
+      "checkWhitelistIPJA3(",
       "reloadGeoIPDBs()",
-      "addCustomStat",
-      "setSiblingsWithKey",
-      "addSiblingWithKey",
-      "removeSibling",
-      "setSiblingConnectTimeout",
-      "setMaxSiblingQueueSize",
-      "setCustomEndpoint",
-      "setCustomGetEndpoint",
-      "setVerboseAllowLog",
-      "incCustomStat"
+      "addCustomStat(",
+      "setSiblingsWithKey(",
+      "addSiblingWithKey(",
+      "removeSibling(",
+      "setSiblingConnectTimeout(",
+      "setMaxSiblingQueueSize(",
+      "setCustomEndpoint(",
+      "setCustomGetEndpoint(",
+      "setVerboseAllowLog(",
+      "incCustomStat("
       };
   static int s_counter=0;
   int counter=0;
